@@ -2,8 +2,7 @@ import logging
 import asyncio
 import json
 
-from copy import copy
-from typing import Optional, Dict, Any, List, TypeVar, Type, Set, Tuple, Union
+from typing import Optional, Dict, Any, List, TypeVar, Type, Set, Tuple, Union, NewType
 from tqdm import tqdm
 
 from abc import ABC
@@ -15,12 +14,16 @@ from .execution_logger import logging_before_and_after, log_error
 logger = logging.getLogger(__name__)
 
 IsResource = TypeVar('IsResource', bound='Resource')
+Alias = NewType('Alias', Union[str, Tuple[str, ...]])
+AliasField = NewType('AliasField', Union[str, Tuple[str, ...]])
 
 
 class ResourceCache:
 
     @logging_before_and_after(logging_level=logger.debug)
-    def __init__(self, resource_class: Type[IsResource], parent: IsResource):
+    def __init__(
+            self, resource_class: Type[IsResource], parent: IsResource
+    ):
         self._parent = parent
         self._resource_class = resource_class
         self._resource_plural = resource_class.plural
@@ -30,7 +33,9 @@ class ResourceCache:
         self._listing_lock = None
 
     @logging_before_and_after(logging_level=logger.debug)
-    async def list(self, limit: Optional[int] = None) -> List[IsResource]:
+    async def list(
+            self, limit: Optional[int] = None
+    ) -> List[IsResource]:
         """ List all child resources
         :param limit: Limit the number of resources to be listed
         :return: List of resources
@@ -45,9 +50,11 @@ class ResourceCache:
                     logger.debug("Resource cache already listed")
                     return list(self._cache.values())
             else:
-                await asyncio.gather(*[resource.update()
-                                       for resource in self._cache.values()
-                                       if 'dirty' in dir(resource) and resource.dirty])
+                await asyncio.gather(
+                    *[resource.update()
+                      for resource in self._cache.values()
+                      if 'dirty' in dir(resource) and resource.dirty]
+                )
                 self._cache = {}
                 self._aliases = {}
 
@@ -57,7 +64,8 @@ class ResourceCache:
 
             resources_raw: Dict = await (
                 self._parent.api_client.query_element(
-                    endpoint=endpoint, method='GET', limit=limit
+                    endpoint=endpoint, method='GET', limit=limit,
+                    elastic_supported=self._resource_class.elastic_supported
                 )
             )
             try:
@@ -68,29 +76,27 @@ class ResourceCache:
                 logger.warning("Resource's json should have an 'items' field")
                 return []
 
+            # Sort resources by id to ensure consistent ordering for alias collision resolution
+            resources = sorted(resources, key=lambda x: x.get('id'))
+
             for resource_dict in resources:
                 resource = self._resource_class(db_resource=resource_dict, parent=self._parent)
-                self._cache[resource_dict.get('id')] = resource
-
-                alias_field = self._resource_class.alias_field
-                alias_entry = None
-                if alias_field in resource_dict:
-                    alias_entry = resource_dict.get(alias_field)
-                elif isinstance(alias_field, Tuple) and alias_field[0] in resource_dict:
-                    alias_entry = resource[alias_field[0]].get(alias_field[1])
-
+                alias_entry = resource['alias']
                 if alias_entry:
+                    if alias_entry in self._aliases:
+                        continue
                     self._aliases[alias_entry] = resource_dict.get('id')
-
+                self._cache[resource_dict.get('id')] = resource
                 resource.empty_changed_params()
 
         self._listing_lock = None
         return list(self._cache.values())
 
     @logging_before_and_after(logging_level=logger.debug)
-    async def add(self, resource: Optional[IsResource] = None, alias: Optional[str] = None,
-                  params: Optional[Dict[str, Any]] = None) -> IsResource:
-
+    async def add(
+            self, resource: Optional[IsResource] = None, alias: Optional[Alias] = None,
+            params: Optional[Dict[str, Any]] = None
+    ) -> IsResource:
         if alias:
             await self.raise_if_alias_exists(alias)
 
@@ -117,7 +123,7 @@ class ResourceCache:
                     del self._aliases[alias_key]
                     break
 
-        alias = resource[resource.alias_field] if not alias and resource.alias_field else alias
+        alias = resource['alias']
 
         if alias is not None:
             self._aliases[alias] = resource_id
@@ -127,17 +133,19 @@ class ResourceCache:
         return resource
 
     @logging_before_and_after(logging_level=logger.debug)
-    async def get(self, uuid: Optional[str] = None, alias: Optional[str] = None) -> Optional[IsResource]:
-
+    async def get(
+            self, uuid: Optional[str] = None, alias: Optional[Alias] = None
+    ) -> Optional[IsResource]:
         if uuid and alias:
             log_error(logger, "Only one of uuid or alias can be provided", CacheError)
 
-        if alias in self._aliases:
-            uuid = self._aliases[alias]
+        if self._parent.api_client.cache_enabled:
+            if alias in self._aliases:
+                uuid = self._aliases[alias]
 
-        if uuid in self._cache and self._parent.api_client.cache_enabled:
-            logger.debug(f"CACHE HIT: Resource {uuid}")
-            return self._cache[uuid]
+            if uuid in self._cache:
+                logger.debug(f"CACHE HIT: Resource {uuid}")
+                return self._cache[uuid]
 
         if not uuid:
             if alias is not None:
@@ -162,19 +170,19 @@ class ResourceCache:
         return await self.add(resource=resource, alias=alias)
 
     @logging_before_and_after(logging_level=logger.debug)
-    async def update(self, uuid: Optional[str] = None, alias: Optional[str] = None, **params
-                     ) -> bool:
-
+    async def update(
+        self, uuid: Optional[str] = None, alias: Optional[Alias] = None, **params
+    ) -> bool:
         resource: IsResource = await self.get(uuid=uuid, alias=alias)
         if not resource:
             return False
 
-        if params.get('new_alias') is not None:
-            await self.raise_if_alias_exists(params['new_alias'])
-            params[resource.alias_field] = params['new_alias']
+        if params.get('new_alias'):
+            new_alias = resource.get_alias_from_other_params(params)
+            await self.raise_if_alias_exists(new_alias)
+            del self._aliases[resource['alias']]
+            self._aliases[new_alias] = resource['id']
             del params['new_alias']
-            del self._aliases[resource[resource.alias_field]]
-            self._aliases[params[resource.alias_field]] = resource['id']
 
         resource.set_params(**params)
         await resource.update()
@@ -182,7 +190,9 @@ class ResourceCache:
         return True
 
     @logging_before_and_after(logging_level=logger.debug)
-    async def delete(self, uuid: Optional[str] = None, alias: Optional[str] = None) -> bool:
+    async def delete(
+            self, uuid: Optional[str] = None, alias: Optional[Alias] = None
+    ) -> bool:
         resource: IsResource = await self.get(uuid=uuid, alias=alias)
         if not resource:
             log_error(logger, f"Resource {uuid} not found in cache, unable to delete it", CacheError)
@@ -194,7 +204,7 @@ class ResourceCache:
         return True
 
     @logging_before_and_after(logging_level=logger.debug)
-    async def raise_if_alias_exists(self, alias: str):
+    async def raise_if_alias_exists(self, alias: Alias):
         await self.list()
         if alias in self._aliases:
             if self._parent.api_client.cache_enabled:
@@ -229,12 +239,13 @@ class ResourceCache:
 
 class BaseResource:
     """ Base class for all the resources in the API """
+
     def __init__(
-            self, parent: Optional[IsResource] = None, uuid: Optional[str] = None,
-            api_client: Optional[ApiClient] = None, params: Optional[Dict[str, Any]] = None,
-            alias_field: Optional[Union[str, Tuple[str, str]]] = None, resource_type: Optional[str] = None,
-            params_to_serialize: Optional[List[str]] = None, await_children: Optional[bool] = False,
-            wrapper_class_instance: Optional[IsResource] = None
+        self, parent: Optional[IsResource] = None, uuid: Optional[str] = None,
+        api_client: Optional[ApiClient] = None, params: Optional[Dict[str, Any]] = None,
+        alias_field: Optional[AliasField] = None, resource_type: Optional[str] = None,
+        params_to_serialize: Optional[List[str]] = None, await_children: Optional[bool] = False,
+        wrapper_class_instance: Optional[IsResource] = None
     ):
         self.alias_field = alias_field
         self.resource_type = resource_type
@@ -257,11 +268,37 @@ class BaseResource:
 
             self.base_url = f'{self.parent["base_url"]}{self.parent.resource_type}/{self.parent["id"]}/'
 
-            self.api_client = self.parent.api_client
+            self.api_client: ApiClient = self.parent.api_client
 
         if not self.api_client:
             log_error(logger, f"Api client has not been defined", ValueError)
 
+    @logging_before_and_after(logging_level=logger.debug)
+    def get_alias_field_value(self, alias_field: str, params: Optional[dict]) -> str:
+        """ Returns the value of one of the fields used to create the alias
+        :param alias_field: The name of the field
+        :param params: The params to use to get the value of the field
+        :return: The value of the field
+        """
+        if not params:
+            params = self.params
+        path = alias_field.split('/')
+        alias = params[path[0]]
+        for path_step in path[1:]:
+            alias = alias[path_step]
+        return alias
+
+    @logging_before_and_after(logging_level=logger.debug)
+    def get_alias(self, params: Optional[dict] = None) -> Optional[Alias]:
+        if self.alias_field:
+            if isinstance(self.alias_field, tuple):
+                return tuple(self.get_alias_field_value(alias_field, params=params) for alias_field in self.alias_field)
+            if isinstance(self.alias_field, str):
+                return self.get_alias_field_value(self.alias_field, params=params)
+            log_error(logger, f"Alias field {self.alias_field} is not a string or a tuple", ValueError)
+        else:
+            return None
+
     @logging_before_and_after(logging_level=logger.debug)
     async def async_init(self):
         """
@@ -291,7 +328,7 @@ class BaseResource:
 
         for field, value in resource_dict.items():
             if field in self.params:
-                self.params[field] = value if field not in self.params_to_serialize\
+                self.params[field] = value if field not in self.params_to_serialize \
                     else json.loads(value if value else '{}')
 
         self.changed_params = set()
@@ -304,15 +341,18 @@ class BaseResource:
 
         self.changed_params = set()
 
-        params = self.params.copy()
+        params = {param: p_value for param, p_value in self.params.copy().items() if p_value is not None}
 
         for field in self.params_to_serialize:
-            if field in self.params:
-                params[field] = json.dumps(params[field])
+            if field in params:
+                try:
+                    params[field] = json.dumps(params[field])
+                except TypeError:
+                    log_error(logger, f"Field {field} is not serializable", TypeError)
 
-            assert isinstance(self.params[field], (dict, list))
+        params = {k: v for k, v in params.items() if v is not None}
 
-        obj = await(
+        obj = await (
             self.api_client.query_element(
                 method='POST', endpoint=endpoint,
                 **{'body_params': params}
@@ -321,7 +361,13 @@ class BaseResource:
 
         for k, v in obj.items():
             if k in self.params:
-                self.params[k] = v if k not in self.params_to_serialize else json.loads(v)
+                if v is None or k not in self.params_to_serialize:
+                    self.params[k] = v
+                else:
+                    try:
+                        self.params[k] = json.loads(v)
+                    except TypeError:
+                        log_error(logger, f"Field {k} is not deserializable", TypeError)
 
         return obj['id']
 
@@ -361,12 +407,15 @@ class BaseResource:
         )
 
     @logging_before_and_after(logging_level=logger.debug)
-    async def create_children_batch(self, resource_class: Type[IsResource], children_params: List[Dict],
-                                    unit: str, batch_size: int = 100):
+    async def create_children_batch(
+        self, resource_class: Type[IsResource], children_params: List[Dict],
+        unit: str, batch_size: int = 100
+    ):
         """ Creates a batch of children of a given resource class. It doesn't return the created resources,
         And it doesn't save them in the cache.
         :param resource_class: The class of the resource to create.
         :param children_params: The parameters of the resources to create.
+        :param unit: The unit of the progress bar.
         :param batch_size: The size of the batch to create.
         """
         if batch_size >= 1000:
@@ -377,6 +426,15 @@ class BaseResource:
         log_level = logger.getEffectiveLevel()
         disable = log_level > logging.INFO or len(children_params) < 1000
 
+        # TODO Seems to be converted in the api when returned? Solve in local server if that is the case
+        if self.api_client.playground:
+            # TODO maybe params_to_serialize should be a class attribute
+            mock_resource = resource_class(parent=self.wrapper_class_instance)
+            for child_params in children_params:
+                for field in mock_resource.params_to_serialize:
+                    if field in child_params:
+                        child_params[field] = json.dumps(child_params[field])
+
         if not disable:
             logger.info("Uploading data")
 
@@ -396,7 +454,9 @@ class BaseResource:
         logger.info("data uploaded") if not disable else None
 
     @logging_before_and_after(logging_level=logger.debug)
-    async def create_child(self, resource_class: Type[IsResource], **kwargs) -> IsResource:
+    async def create_child(
+            self, resource_class: Type[IsResource], **kwargs
+    ) -> IsResource:
         """ Creates a child of a given resource class.
         :param resource_class: The class of the resource to create.
         :param kwargs: The parameters of the resource to create.
@@ -421,9 +481,11 @@ class BaseResource:
         return await res_cache.add(**kwargs)
 
     @logging_before_and_after(logging_level=logger.debug)
-    async def update_child(self, resource_class: Type[IsResource],
-                           uuid: Optional[str] = None, alias: Optional[str] = None,
-                           **params) -> bool:
+    async def update_child(
+        self, resource_class: Type[IsResource],
+        uuid: Optional[str] = None, alias: Optional[Alias] = None,
+        **params
+    ) -> bool:
         """ Updates a child of a given resource class.
         :param resource_class: The class of the resource to update.
         :param uuid: The uuid of the resource to update.
@@ -434,7 +496,9 @@ class BaseResource:
         return await self.children[resource_class].update(uuid=uuid, alias=alias, **params)
 
     @logging_before_and_after(logging_level=logger.debug)
-    async def get_children(self, resource_class: Type[IsResource], limit: Optional[int] = None) -> List[IsResource]:
+    async def get_children(
+            self, resource_class: Type[IsResource], limit: Optional[int] = None
+    ) -> List[IsResource]:
         """ Gets all children of a given resource class.
         :param resource_class: The class of the resources to get.
         :param limit: The maximum number of resources to get.
@@ -444,9 +508,9 @@ class BaseResource:
 
     @logging_before_and_after(logging_level=logger.debug)
     async def get_child(
-        self, resource_class: Type[IsResource],
-        uuid: Optional[str] = None, alias: Optional[str] = None,
-        create_if_not_exists: bool = False
+            self, resource_class: Type[IsResource],
+            uuid: Optional[str] = None, alias: Optional[Alias] = None,
+            create_if_not_exists: bool = False
     ) -> Optional[IsResource]:
         """ Gets a child resource.
         :param resource_class: The class of the resource to get.
@@ -467,8 +531,10 @@ class BaseResource:
         return child
 
     @logging_before_and_after(logging_level=logger.debug)
-    async def delete_child(self, resource_class: Type[IsResource],
-                           uuid: Optional[str] = None, alias: Optional[str] = None) -> bool:
+    async def delete_child(
+        self, resource_class: Type[IsResource],
+        uuid: Optional[str] = None, alias: Optional[Alias] = None
+    ) -> bool:
         """ Deletes a child resource.
         :param resource_class: The class of the resource to delete.
         :param uuid: The uuid of the resource to delete.
@@ -499,12 +565,7 @@ class BaseResource:
 
     def __str__(self):
         """ Returns a string representation of the resource. """
-        alias = None
-        if self.alias_field:
-            if isinstance(self.alias_field, Tuple):
-                alias = self.params[self.alias_field[0]][self.alias_field[1]]
-            else:
-                alias = self.params[self.alias_field]
+        alias = self.get_alias()
         return self.id if not alias else alias
 
     def __eq__(self, other):
@@ -513,22 +574,22 @@ class BaseResource:
 
 
 class Resource(ABC):
-
-    alias_field: Optional[str] = None
+    alias_field: Optional[AliasField] = None
     resource_type: Optional[str] = None
     plural: Optional[str] = None
+    elastic_supported: bool = False
 
     @logging_before_and_after(logging_level=logger.debug)
     def __init__(
-        self, parent: Optional[IsResource] = None,
-        uuid: Optional[str] = None,
-        db_resource: Optional[dict[str, Any]] = None,
-        api_client: Optional[ApiClient] = None,
-        params: Optional[dict[str, Any]] = None,
-        children: Optional[List[Type[IsResource]]] = None,
-        check_params_before_creation: Optional[List[str]] = None,
-        params_to_serialize: Optional[List[str]] = None,
-        await_children: Optional[bool] = False
+            self, parent: Optional[IsResource] = None,
+            uuid: Optional[str] = None,
+            db_resource: Optional[dict[str, Any]] = None,
+            api_client: Optional[ApiClient] = None,
+            params: Optional[dict[str, Any]] = None,
+            children: Optional[List[Type[IsResource]]] = None,
+            check_params_before_creation: Optional[List[str]] = None,
+            params_to_serialize: Optional[List[str]] = None,
+            await_children: Optional[bool] = False
     ):
         """ Initializes a resource. """
 
@@ -537,23 +598,27 @@ class Resource(ABC):
         else:
             db_resource = {}
 
-        self._base_resource = BaseResource(api_client=api_client, uuid=uuid, parent=parent, params=params,
-                                           resource_type=self.resource_type, alias_field=self.alias_field,
-                                           params_to_serialize=params_to_serialize, await_children=await_children,
-                                           wrapper_class_instance=self)
+        self._base_resource = BaseResource(
+            api_client=api_client, uuid=uuid, parent=parent, params=params,
+            resource_type=self.resource_type, alias_field=self.alias_field,
+            params_to_serialize=params_to_serialize, await_children=await_children,
+            wrapper_class_instance=self
+        )
 
         self._base_resource.children = {
             res_class: ResourceCache(res_class, self) for res_class in children
         } if children else {}
 
         self._check_params_before_creation = check_params_before_creation if check_params_before_creation else []
-        self.api_client = self._base_resource.api_client
+        self.api_client: ApiClient = self._base_resource.api_client
+        self.parent = self._base_resource.parent
         self.params_to_serialize = self._base_resource.params_to_serialize
 
         for field, value in db_resource.items():
             if field in self:
-                self[field] = value if field not in self.params_to_serialize \
-                    else json.loads(value if value else f'{self[field].__class__()}')
+                if field in self.params_to_serialize and value is not None:
+                    value = json.loads(value)
+                self[field] = value
 
     @logging_before_and_after(logging_level=logger.debug)
     def __await__(self):
@@ -593,17 +658,30 @@ class Resource(ABC):
         """ Empties the changed_params set. """
         self._base_resource.changed_params = set()
 
+    @logging_before_and_after(logging_level=logger.debug)
+    def get_alias_from_other_params(self, params: dict) -> Optional[str]:
+        """ Returns the alias of the resource based on the given parameters. """
+        return self._base_resource.get_alias(params)
+
     def __getitem__(self, item):
         """ Returns the value of the parameter with the given name. """
         if item == 'id':
             return self._base_resource.id
         elif item == 'base_url':
             return self._base_resource.base_url
+        elif item == 'alias':
+            return self._base_resource.get_alias()
         elif item in self._base_resource.params:
             result = self._base_resource.params[item]
             if isinstance(result, (dict, list)):
                 self._base_resource.changed_params.add(item)
             return result
+        elif isinstance(item, tuple) and len(item) == 2 and isinstance(item[0], str) and isinstance(item[1], str) and \
+                item[0] in self._base_resource.params:
+            deeper_params = self._base_resource.params[item[0]]
+            if item[1] not in deeper_params:
+                log_error(logger, f"{self.__class__.__name__} {item[0]} does not contain {item[1]}", KeyError)
+            return deeper_params[item[1]]
         else:
             log_error(logger, f"{self.__class__.__name__} parameters do not contain {item}", KeyError)
 
